searchNodes=[{"ref":"Ecto.Adapters.SQL.html","title":"Ecto.Adapters.SQL","type":"module","doc":"This application provides functionality for working with SQL databases in Ecto. Built-in adapters By default, we support the following adapters: Ecto.Adapters.Postgres for Postgres Ecto.Adapters.MyXQL for MyXQL Ecto.Adapters.MySQL for MySQL (legacy - it will be removed in future versions) Migrations Ecto supports database migrations. You can generate a migration with: $ mix ecto.gen.migration create_posts This will create a new file inside priv/repo/migrations with the change function. Check Ecto.Migration for more information. To interface with migrations, developers typically use mix tasks: mix ecto.migrations - lists all available migrations and their status mix ecto.migrate - runs a migration mix ecto.rollback - rolls back a previously run migration If you want to run migrations programatically, see Ecto.Migrator. SQL sandbox ecto_sql provides a sandbox for testing. The sandbox wraps each test in a transaction, making sure the tests are isolated and can run concurrently. See Ecto.Adapters.SQL.Sandbox for more information. Structure load and dumping If you have an existing database, you may want to dump its existing structure and make it reproducible from within Ecto. This can be achieved with two Mix tasks: mix ecto.load - loads an existing structure into the database mix ecto.dump - dumps the existing database structure to the filesystem For creating and dropping databases, see mix ecto.create and mix ecto.drop that are included as part of Ecto. Custom adapters Developers can implement their own SQL adapters by using Ecto.Adapters.SQL and by implementing the callbacks required by Ecto.Adapters.SQL.Connection for handling connections and performing queries. The connection handling and pooling for SQL adapters should be built using the DBConnection library. When using Ecto.Adapters.SQL, the following options are required: :driver (required) - the database driver library. For example: :postgrex :migration_lock - the lock to use on migration locks. For example: &quot;FOR UPDATE&quot;. It may also be nil (for no lock). The user can still override this by setting :migration_lock in the repository configuration"},{"ref":"Ecto.Adapters.SQL.html#query/4","title":"Ecto.Adapters.SQL.query/4","type":"function","doc":"Runs custom SQL query on given repo. In case of success, it must return an :ok tuple containing a map with at least two keys: :num_rows - the number of rows affected :rows - the result set as a list. nil may be returned instead of the list if the command does not yield any row as result (but still yields the number of affected rows, like a delete command without returning would) Options :log - When false, does not log the query Examples iex&gt; Ecto.Adapters.SQL.query(MyRepo, &quot;SELECT $1::integer + $2&quot;, [40, 2]) {:ok, %{rows: [[42]], num_rows: 1}} For convenience, this function is also available under the repository: iex&gt; MyRepo.query(&quot;SELECT $1::integer + $2&quot;, [40, 2]) {:ok, %{rows: [[42]], num_rows: 1}}"},{"ref":"Ecto.Adapters.SQL.html#query!/4","title":"Ecto.Adapters.SQL.query!/4","type":"function","doc":"Same as query/4 but raises on invalid queries."},{"ref":"Ecto.Adapters.SQL.html#stream/4","title":"Ecto.Adapters.SQL.stream/4","type":"function","doc":"Returns a stream that runs a custom SQL query on given repo when reduced. In case of success it is a enumerable containing maps with at least two keys: :num_rows - the number of rows affected :rows - the result set as a list. nil may be returned instead of the list if the command does not yield any row as result (but still yields the number of affected rows, like a delete command without returning would) In case of failure it raises an exception. If the adapter supports a collectable stream, the stream may also be used as the collectable in Enum.into/3. Behaviour depends on the adapter. Options :log - When false, does not log the query :max_rows - The number of rows to load from the database as we stream Examples iex&gt; Ecto.Adapters.SQL.stream(MyRepo, &quot;SELECT $1::integer + $2&quot;, [40, 2]) |&gt; Enum.to_list() [%{rows: [[42]], num_rows: 1}]"},{"ref":"Ecto.Adapters.SQL.html#table_exists?/2","title":"Ecto.Adapters.SQL.table_exists?/2","type":"function","doc":"Check if the given table exists. Returns true if the table exists in the repo, otherwise false. The table is checked against the current database/schema in the connection."},{"ref":"Ecto.Adapters.SQL.html#to_sql/3","title":"Ecto.Adapters.SQL.to_sql/3","type":"function","doc":"Converts the given query to SQL according to its kind and the adapter in the given repository. Examples The examples below are meant for reference. Each adapter will return a different result: iex&gt; Ecto.Adapters.SQL.to_sql(:all, repo, Post) {&quot;SELECT p.id, p.title, p.inserted_at, p.created_at FROM posts as p&quot;, []} iex&gt; Ecto.Adapters.SQL.to_sql(:update_all, repo, from(p in Post, update: [set: [title: ^&quot;hello&quot;]])) {&quot;UPDATE posts AS p SET title = $1&quot;, [&quot;hello&quot;]} This function is also available under the repository with name to_sql: iex&gt; Repo.to_sql(:all, Post) {&quot;SELECT p.id, p.title, p.inserted_at, p.created_at FROM posts as p&quot;, []}"},{"ref":"Ecto.Adapters.SQL.Sandbox.html","title":"Ecto.Adapters.SQL.Sandbox","type":"module","doc":"A pool for concurrent transactional tests. The sandbox pool is implemented on top of an ownership mechanism. When started, the pool is in automatic mode, which means the repository will automatically check connections out as with any other pool. The mode/2 function can be used to change the pool mode to manual or shared. In both modes, the connection must be explicitly checked out before use. When explicit checkouts are made, the sandbox will wrap the connection in a transaction by default and control who has access to it. This means developers have a safe mechanism for running concurrent tests against the database. Database support While both PostgreSQL and MySQL support SQL Sandbox, only PostgreSQL supports concurrent tests while running the SQL Sandbox. Therefore, do not run concurrent tests with MySQL as you may run into deadlocks due to its transaction implementation. Example The first step is to configure your database to use the Ecto.Adapters.SQL.Sandbox pool. You set those options in your config/config.exs (or preferably config/test.exs) if you haven&#39;t yet: config :my_app, Repo, pool: Ecto.Adapters.SQL.Sandbox Now with the test database properly configured, you can write transactional tests: # At the end of your test_helper.exs # Set the pool mode to manual for explicit checkouts Ecto.Adapters.SQL.Sandbox.mode(Repo, :manual) defmodule PostTest do # Once the mode is manual, tests can also be async use ExUnit.Case, async: true setup do # Explicitly get a connection before each test :ok = Ecto.Adapters.SQL.Sandbox.checkout(Repo) end test &quot;create post&quot; do # Use the repository as usual assert %Post{} = Repo.insert!(%Post{}) end end Collaborating processes The example above is straight-forward because we have only a single process using the database connection. However, sometimes a test may need to interact with multiple processes, all using the same connection so they all belong to the same transaction. Before we discuss solutions, let&#39;s see what happens if we try to use a connection from a new process without explicitly checking it out first: setup do # Explicitly get a connection before each test :ok = Ecto.Adapters.SQL.Sandbox.checkout(Repo) end test &quot;calls worker that runs a query&quot; do GenServer.call(MyApp.Worker, :run_query) end The test above will fail with an error similar to: ** (DBConnection.OwnershipError) cannot find ownership process for #PID&lt;0.35.0&gt; That&#39;s because the setup block is checking out the connection only for the test process. Once the worker attempts to perform a query, there is no connection assigned to it and it will fail. The sandbox module provides two ways of doing so, via allowances or by running in shared mode. Allowances The idea behind allowances is that you can explicitly tell a process which checked out connection it should use, allowing multiple processes to collaborate over the same connection. Let&#39;s give it a try: test &quot;calls worker that runs a query&quot; do Ecto.Adapters.SQL.Sandbox.allow(Repo, self(), MyApp.Worker) GenServer.call(MyApp.Worker, :run_query) end And that&#39;s it, by calling allow/3, we are explicitly assigning the parent&#39;s connection (i.e. the test process&#39; connection) to the task. Because allowances use an explicit mechanism, their advantage is that you can still run your tests in async mode. The downside is that you need to explicitly control and allow every single process. This is not always possible. In such cases, you will want to use shared mode. Shared mode Shared mode allows a process to share its connection with any other process automatically, without relying on explicit allowances. Let&#39;s change the example above to use shared mode: setup do # Explicitly get a connection before each test :ok = Ecto.Adapters.SQL.Sandbox.checkout(Repo) # Setting the shared mode must be done only after checkout Ecto.Adapters.SQL.Sandbox.mode(Repo, {:shared, self()}) end test &quot;calls worker that runs a query&quot; do GenServer.call(MyApp.Worker, :run_query) end By calling mode({:shared, self()}), any process that needs to talk to the database will now use the same connection as the one checked out by the test process during the setup block. Make sure to always check a connection out before setting the mode to {:shared, self()}. The advantage of shared mode is that by calling a single function, you will ensure all upcoming processes and operations will use that shared connection, without a need to explicitly allow them. The downside is that tests can no longer run concurrently in shared mode. Also, beware that if the test process terminates while the worker is using the connection, the connection will be taken away from the worker, which will error. Therefore it is important to guarantee the work is done before the test concludes. In the example above, we are using a call, which is synchronous, avoiding the problem, but you may need to explicitly flush the worker or terminate it under such scenarios in your tests. Summing up There are two mechanisms for explicit ownerships: Using allowances - requires explicit allowances via allow/3. Tests may run concurrently. Using shared mode - does not require explicit allowances. Tests cannot run concurrently. FAQ When running the sandbox mode concurrently, developers may run into issues we explore in the upcoming sections. &quot;owner exited while client is still running&quot; In some situations, you may see error reports similar to the one below: 21:57:43.910 [error] Postgrex.Protocol (#PID&lt;0.284.0&gt;) disconnected: ** (DBConnection.Error) owner #PID&lt;&gt; exited while client #PID&lt;&gt; is still running Such errors are usually followed by another error report from another process that failed while executing a database query. To understand the failure, we need to answer the question: who are the owner and client processes? The owner process is the one that checks out the connection, which, in the majority of cases, is the test process, the one running your tests. In other words, the error happens because the test process has finished, either because the test succeeded or because it failed, while the client process was trying to get information from the database. Since the owner process, the one that owns the connection, no longer exists, Ecto will check the connection back in and notify the client process using the connection that the connection owner is no longer available. This can happen in different situations. For example, imagine you query a GenServer in your test that is using a database connection: test &quot;gets results from GenServer&quot; do {:ok, pid} = MyAppServer.start_link() Ecto.Adapters.SQL.Sandbox.allow(Repo, self(), pid) assert MyAppServer.get_my_data_fast(timeout: 1000) == [...] end In the test above, we spawn the server and allow it to perform database queries using the connection owned by the test process. Since we gave a timeout of 1 second, in case the database takes longer than one second to reply, the test process will fail, due to the timeout, making the &quot;owner down&quot; message to be printed because the server process is still waiting on a connection reply. In some situations, such failures may be intermittent. Imagine that you allow a process that queries the database every half second: test &quot;queries periodically&quot; do {:ok, pid} = PeriodicServer.start_link() Ecto.Adapters.SQL.Sandbox.allow(Repo, self(), pid) # more tests end Because the server is querying the database from time to time, there is a chance that, when the test exits, the periodic process may be querying the database, regardless of test success or failure. &quot;owner timed out because it owned the connection for longer than Nms&quot; In some situations, you may see error reports similar to the one below: 09:56:43.081 [error] Postgrex.Protocol (#PID&lt;&gt;) disconnected: ** (DBConnection.ConnectionError) owner #PID&lt;&gt; timed out because it owned the connection for longer than 60000ms If you have a long running test (or you&#39;re debugging with IEx.pry), the timeout for the connection ownership may be too short. You can increase the timeout by setting the :ownership_timeout options for your repo config in config/config.exs (or preferably in config/test.exs): config :my_app, MyApp.Repo, ownership_timeout: NEW_TIMEOUT_IN_MILLISECONDS The :ownership_timeout option is part of DBConnection.Ownership and defaults to 60000ms. Timeouts are given as integers in milliseconds. Alternately, if this is an issue for only a handful of long-running tests, you can pass an :ownership_timeout option when calling Ecto.Adapters.SQL.Sandbox.checkout/2 instead of setting a longer timeout globally in your config. Deferred constraints Some databases allow to defer constraint validation to the transaction commit time, instead of the particular statement execution time. This feature, for instance, allows for a cyclic foreign key referencing. Since the SQL Sandbox mode rolls back transactions, tests might report false positives because deferred constraints are never checked by the database. To manually force deferred constraints validation when using PostgreSQL use the following line right at the end of your test case: Repo.query!(&quot;SET CONSTRAINTS ALL IMMEDIATE&quot;) Database locks and deadlocks Since the sandbox relies on concurrent transactional tests, there is a chance your tests may trigger deadlocks in your database. This is specially true with MySQL, where the solutions presented here are not enough to avoid deadlocks and therefore making the use of concurrent tests with MySQL prohibited. However, even on databases like PostgreSQL, performance degradations or deadlocks may still occur. For example, imagine multiple tests are trying to insert the same user to the database. They will attempt to retrieve the same database lock, causing only one test to succeed and run while all other tests wait for the lock. In other situations, two different tests may proceed in a way that each test retrieves locks desired by the other, leading to a situation that cannot be resolved, a deadlock. For instance: Transaction 1: Transaction 2: begin begin update posts where id = 1 update posts where id = 2 update posts where id = 1 update posts where id = 2 **deadlock** There are different ways to avoid such problems. One of them is to make sure your tests work on distinct data. Regardless of your choice between using fixtures or factories for test data, make sure you get a new set of data per test. This is specially important for data that is meant to be unique like user emails. For example, instead of: def insert_user do Repo.insert! %User{email: &quot;sample@example.com&quot;} end prefer: def insert_user do Repo.insert! %User{email: &quot;sample-\#{counter()}@example.com&quot;} end defp counter do System.unique_integer [:positive] end In fact, avoiding unique emails like above can also have a positive impact on the test suite performance, as it reduces contention and wait between concurrent tests. We have heard reports where using dynamic values for uniquely indexed columns, as we did for e-mail above, made a test suite run between 2x to 3x faster. Deadlocks may happen in other circumstances. If you believe you are hitting a scenario that has not been described here, please report an issue so we can improve our examples. As a last resort, you can always disable the test triggering the deadlock from running asynchronously by setting &quot;async: false&quot;."},{"ref":"Ecto.Adapters.SQL.Sandbox.html#allow/4","title":"Ecto.Adapters.SQL.Sandbox.allow/4","type":"function","doc":"Allows the allow process to use the same connection as parent."},{"ref":"Ecto.Adapters.SQL.Sandbox.html#checkin/2","title":"Ecto.Adapters.SQL.Sandbox.checkin/2","type":"function","doc":"Checks in the connection back into the sandbox pool."},{"ref":"Ecto.Adapters.SQL.Sandbox.html#checkout/2","title":"Ecto.Adapters.SQL.Sandbox.checkout/2","type":"function","doc":"Checks a connection out for the given repo. The process calling checkout/2 will own the connection until it calls checkin/2 or until it crashes when then the connection will be automatically reclaimed by the pool. Options :sandbox - when true the connection is wrapped in a transaction. Defaults to true. :isolation - set the query to the given isolation level. :ownership_timeout - limits how long the connection can be owned. Defaults to the value in your repo config in config/config.exs (or preferably in config/test.exs), or 60000 ms if not set. The timeout exists for sanity checking purposes, to ensure there is no connection leakage, and can be bumped whenever necessary."},{"ref":"Ecto.Adapters.SQL.Sandbox.html#mode/2","title":"Ecto.Adapters.SQL.Sandbox.mode/2","type":"function","doc":"Sets the mode for the repo pool. The mode can be :auto, :manual or {:shared, &lt;pid&gt;}. Warning: you should only call this function in the setup block for a test and not within a test, because if the mode is changed during the test it will cause other database connections to be checked in (causing errors)."},{"ref":"Ecto.Adapters.SQL.Sandbox.html#unboxed_run/2","title":"Ecto.Adapters.SQL.Sandbox.unboxed_run/2","type":"function","doc":"Runs a function outside of the sandbox."},{"ref":"Ecto.Migration.html","title":"Ecto.Migration","type":"behaviour","doc":"Migrations are used to modify your database schema over time. This module provides many helpers for migrating the database, allowing developers to use Elixir to alter their storage in a way that is database independent. Here is an example: defmodule MyRepo.Migrations.AddWeatherTable do use Ecto.Migration def up do create table(&quot;weather&quot;) do add :city, :string, size: 40 add :temp_lo, :integer add :temp_hi, :integer add :prcp, :float timestamps() end end def down do drop table(&quot;weather&quot;) end end Note that migrations have up/0 and down/0 instructions, where up/0 applies changes to the database and down/0 rolls back changes, returning the database schema to a previous state. Ecto creates a table (see the :migration_source configuration option) in the database in order to keep track of migrations and will add an entry to this table for each migration you define. Ecto also locks the table when adding/removing entries, guaranteeing two different servers cannot run the same migration at the same time. Ecto provides some mix tasks to help developers work with migrations: mix ecto.gen.migration - generates a migration that the user can fill in with particular commands mix ecto.migrate - migrates a repository mix ecto.rollback - rolls back a particular migration Run mix help COMMAND for more information on a particular command. Change change/0 is an abstraction that wraps both up/0 and down/0 for automatically-reversible migrations. For example, the migration above can be written as: defmodule MyRepo.Migrations.AddWeatherTable do use Ecto.Migration def change do create table(&quot;weather&quot;) do add :city, :string, size: 40 add :temp_lo, :integer add :temp_hi, :integer add :prcp, :float timestamps() end end end However, note that not all commands are reversible. Trying to rollback a non-reversible command will raise an Ecto.MigrationError. A notable command in this regard is execute/2, which is reversible in change/0 by accepting a pair of plain SQL strings. The first is run on forward migrations (up/0) and the second when rolling back (down/0). If up/0 and down/0 are implemented in a migration, they take precedence, and change/0 isn&#39;t invoked. Field Types The Ecto primitive types are mapped to the appropriate database type by the various database adapters. For example, :string is converted to :varchar, :binary to :bits or :blob, and so on. Similarly, you can pass any field type supported by your database as long as it maps to an Ecto type. For instance, you can use :text, :varchar, or :char in your migrations as add :field_name, :text. In your Ecto schema, they will all map to the same :string type. Remember, atoms can contain arbitrary characters by enclosing in double quotes the characters following the colon. So, if you want to use a field type with database-specific options, you can pass atoms containing these options like :&quot;int unsigned&quot;, :&quot;time without time zone&quot;, etc. Prefixes Migrations support specifying a table prefix or index prefix which will target either a schema (if using PostgreSQL) or a different database (if using MySQL). If no prefix is provided, the default schema or database is used. Any reference declared in the table migration refers by default to the table with the same declared prefix. The prefix is specified in the table options: def up do create table(&quot;weather&quot;, prefix: &quot;north_america&quot;) do add :city, :string, size: 40 add :temp_lo, :integer add :temp_hi, :integer add :prcp, :float add :group_id, references(:groups) timestamps() end create index(&quot;weather&quot;, [:city], prefix: &quot;north_america&quot;) end Note: if using MySQL with a prefixed table, you must use the same prefix for the references since cross-database references are not supported. When using a prefixed table with either MySQL or PostgreSQL, you must use the same prefix for the index field to ensure that you index the prefix-qualified table. Transactions For PostgreSQL, Ecto always runs migrations inside a transaction, but that&#39;s not always desired: for example, you cannot create/drop indexes concurrently inside a transaction. Migrations can be forced to run outside a transaction by setting the @disable_ddl_transaction module attribute to true. See the section about concurrent in index/3 for more information. Transaction Callbacks There are use cases that dictate adding some common behavior after beginning a migration transaction, or before commiting that transaction. For instance, one might desire to set a lock_timeout for each lock in the transaction. Another way these might be leveraged is by defining a custom migration module so that these callbacks will run for all of your migrations, if you have special requirements. defmodule MyApp.Migration do defmacro __using__(_) do quote do use Ecto.Migration def after_begin() do repo().query! &quot;SET lock_timeout TO &#39;5s&#39;&quot;, &quot;SET lock_timeout TO &#39;10s&#39;&quot; end end end end Then in your migrations you can use MyApp.Migration to share this behavior among all your migrations. Comments Migrations where you create or alter a table support specifying table and column comments. The same can be done when creating constraints and indexes. Not all databases support this feature. def up do create index(&quot;posts&quot;, [:name], comment: &quot;Index Comment&quot;) create constraint(&quot;products&quot;, &quot;price_must_be_positive&quot;, check: &quot;price &gt; 0&quot;, comment: &quot;Constraint Comment&quot;) create table(&quot;weather&quot;, prefix: &quot;north_america&quot;, comment: &quot;Table Comment&quot;) do add :city, :string, size: 40, comment: &quot;Column Comment&quot; timestamps() end end Repo configuration The following migration configuration options are available for a given repository: :migration_source - Version numbers of migrations will be saved in a table named schema_migrations by default. You can configure the name of the table via: config :app, App.Repo, migration_source: &quot;my_migrations&quot; :migration_primary_key - By default, Ecto uses the :id column with type :bigserial, but you can configure it via: config :app, App.Repo, migration_primary_key: [name: :uuid, type: :binary_id] :migration_timestamps - By default, Ecto uses the :naive_datetime type, but you can configure it via: config :app, App.Repo, migration_timestamps: [type: :utc_datetime] :migration_lock - By default, Ecto will lock the migration table. This allows multiple nodes to attempt to run migrations at the same time but only one will succeed. However, this does not play well with other features, such as the :concurrently option in PostgreSQL indexes. You can disable the migration_lock by setting it to nil: config :app, App.Repo, migration_lock: nil :migration_default_prefix - Ecto defaults to nil for the database prefix for migrations, but you can configure it via: config :app, App.Repo, migration_default_prefix: &quot;my_prefix&quot; :start_apps_before_migration - A list of applications to be started before running migrations. Used by Ecto.Migrator.with_repo/3 and the migration tasks: config :app, App.Repo, start_apps_before_migration: [:ssl, :some_custom_logger]"},{"ref":"Ecto.Migration.html#add/3","title":"Ecto.Migration.add/3","type":"function","doc":"Adds a column when creating or altering a table. This function also accepts Ecto primitive types as column types that are normalized by the database adapter. For example, :string is converted to :varchar, :binary to :bits or :blob, and so on. However, the column type is not always the same as the type used in your schema. For example, a schema that has a :string field can be supported by columns of type :char, :varchar, :text, and others. For this reason, this function also accepts :text and other type annotations that are native to the database. These are passed to the database as-is. To sum up, the column type may be either an Ecto primitive type, which is normalized in cases where the database does not understand it, such as :string or :binary, or a database type which is passed as-is. Custom Ecto types like Ecto.UUID are not supported because they are application-level concerns and may not always map to the database. Examples create table(&quot;posts&quot;) do add :title, :string, default: &quot;Untitled&quot; end alter table(&quot;posts&quot;) do add :summary, :text # Database type add :object, :map # Elixir type which is handled by the database end Options :primary_key - when true, marks this field as the primary key. If multiple fields are marked, a composite primary key will be created. :default - the column&#39;s default value. It can be a string, number, empty list, list of strings, list of numbers, or a fragment generated by fragment/1. :null - when false, the column does not allow null values. :size - the size of the type (for example, the number of characters). The default is no size, except for :string, which defaults to 255. :precision - the precision for a numeric type. Required when :scale is specified. :scale - the scale of a numeric type. Defaults to 0."},{"ref":"Ecto.Migration.html#add_if_not_exists/3","title":"Ecto.Migration.add_if_not_exists/3","type":"function","doc":"Adds a column if it not exists yet when altering a table. If the type value is a %Reference{}, it is used to add a constraint. type and opts are exactly the same as in add/3. This command is not reversible as Ecto does not know about column existense before the creation attempt. Examples alter table(&quot;posts&quot;) do add_if_not_exists :title, :string, default: &quot;&quot; end"},{"ref":"Ecto.Migration.html#c:after_begin/0","title":"Ecto.Migration.after_begin/0","type":"callback","doc":"Migration code to run immediately after the transaction is opened. Keep in mind that it is treated like any normal migration code, and should consider both the up and down cases of the migration."},{"ref":"Ecto.Migration.html#alter/2","title":"Ecto.Migration.alter/2","type":"macro","doc":"Alters a table. Examples alter table(&quot;posts&quot;) do add :summary, :text modify :title, :text remove :views end"},{"ref":"Ecto.Migration.html#c:before_commit/0","title":"Ecto.Migration.before_commit/0","type":"callback","doc":"Migration code to run immediately before the transaction is closed. Keep in mind that it is treated like any normal migration code, and should consider both the up and down cases of the migration."},{"ref":"Ecto.Migration.html#constraint/3","title":"Ecto.Migration.constraint/3","type":"function","doc":"Defines a constraint (either a check constraint or an exclusion constraint) to be evaluated by the database when a row is inserted or updated. Examples create constraint(&quot;users&quot;, :price_must_be_positive, check: &quot;price &gt; 0&quot;) create constraint(&quot;size_ranges&quot;, :no_overlap, exclude: ~s|gist (int4range(&quot;from&quot;, &quot;to&quot;, &#39;[]&#39;) WITH &amp;&amp;)|) drop constraint(&quot;products&quot;, &quot;price_must_be_positive&quot;) Options :check - A check constraint expression. Required when creating a check constraint. :exclude - An exclusion constraint expression. Required when creating an exclusion constraint. :prefix - The prefix for the table."},{"ref":"Ecto.Migration.html#create/1","title":"Ecto.Migration.create/1","type":"function","doc":"Creates one of the following: an index a table with only an :id field a constraint When reversing (in a change/0 running backwards), indexes are only dropped if they exist, and no errors are raised. To enforce dropping an index, use drop/1. Examples create index(&quot;posts&quot;, [:name]) create table(&quot;version&quot;) create constraint(&quot;products&quot;, &quot;price_must_be_positive&quot;, check: &quot;price &gt; 0&quot;)"},{"ref":"Ecto.Migration.html#create/2","title":"Ecto.Migration.create/2","type":"macro","doc":"Creates a table. By default, the table will also include an :id primary key field that has a type of :bigserial. Check the table/2 docs for more information. Examples create table(:posts) do add :title, :string, default: &quot;Untitled&quot; add :body, :text timestamps() end"},{"ref":"Ecto.Migration.html#create_if_not_exists/1","title":"Ecto.Migration.create_if_not_exists/1","type":"function","doc":"Creates an index or a table with only :id field if one does not yet exist. Examples create_if_not_exists index(&quot;posts&quot;, [:name]) create_if_not_exists table(&quot;version&quot;)"},{"ref":"Ecto.Migration.html#create_if_not_exists/2","title":"Ecto.Migration.create_if_not_exists/2","type":"macro","doc":"Creates a table if it does not exist. Works just like create/2 but does not raise an error when the table already exists."},{"ref":"Ecto.Migration.html#direction/0","title":"Ecto.Migration.direction/0","type":"function","doc":"Gets the migrator direction."},{"ref":"Ecto.Migration.html#drop/1","title":"Ecto.Migration.drop/1","type":"function","doc":"Drops one of the following: an index a table a constraint Examples drop index(&quot;posts&quot;, [:name]) drop table(&quot;posts&quot;) drop constraint(&quot;products&quot;, &quot;price_must_be_positive&quot;)"},{"ref":"Ecto.Migration.html#drop_if_exists/1","title":"Ecto.Migration.drop_if_exists/1","type":"function","doc":"Drops a table or index if it exists. Does not raise an error if the specified table or index does not exist. Examples drop_if_exists index(&quot;posts&quot;, [:name]) drop_if_exists table(&quot;posts&quot;)"},{"ref":"Ecto.Migration.html#execute/1","title":"Ecto.Migration.execute/1","type":"function","doc":"Executes arbitrary SQL or a keyword command. Reversible commands can be defined by calling execute/2. Examples execute &quot;CREATE EXTENSION postgres_fdw&quot; execute create: &quot;posts&quot;, capped: true, size: 1024"},{"ref":"Ecto.Migration.html#execute/2","title":"Ecto.Migration.execute/2","type":"function","doc":"Executes reversible SQL commands. This is useful for database-specific functionality that does not warrant special support in Ecto, for example, creating and dropping a PostgreSQL extension. The execute/2 form avoids having to define separate up/0 and down/0 blocks that each contain an execute/1 expression. Examples execute &quot;CREATE EXTENSION postgres_fdw&quot;, &quot;DROP EXTENSION postgres_fdw&quot;"},{"ref":"Ecto.Migration.html#flush/0","title":"Ecto.Migration.flush/0","type":"function","doc":"Executes queue migration commands. Reverses the order in which commands are executed when doing a rollback on a change/0 function and resets the commands queue."},{"ref":"Ecto.Migration.html#fragment/1","title":"Ecto.Migration.fragment/1","type":"function","doc":"Generates a fragment to be used as a default value. Examples create table(&quot;posts&quot;) do add :inserted_at, :naive_datetime, default: fragment(&quot;now()&quot;) end"},{"ref":"Ecto.Migration.html#index/3","title":"Ecto.Migration.index/3","type":"function","doc":"Returns an index struct that can be given to create/1, drop/1, etc. Expects the table name as the first argument and the index field(s) as the second. The fields can be atoms, representing columns, or strings, representing expressions that are sent as-is to the database. Options :name - the name of the index. Defaults to &quot;\#{table}_\#{column}_index&quot;. :unique - indicates whether the index should be unique. Defaults to false. :concurrently - indicates whether the index should be created/dropped concurrently. :using - configures the index type. :prefix - specify an optional prefix for the index. :where - specify conditions for a partial index. Adding/dropping indexes concurrently PostgreSQL supports adding/dropping indexes concurrently (see the docs). However, this feature does not work well with the transactions used by Ecto to guarantee integrity during migrations. Therefore, to migrate indexes concurrently, you need to set @disable_ddl_transaction in the migration to true, disabling the guarantee that all of the changes in the migration will happen at once: defmodule MyRepo.Migrations.CreateIndexes do use Ecto.Migration @disable_ddl_transaction true def change do create index(&quot;posts&quot;, [:slug], concurrently: true) end end And you also need to disable the migration lock for that repository: config :my_app, MyApp.Repo, migration_lock: nil The migration lock is used to guarantee that only one node in a cluster can run migrations. Two nodes may attempt to race each other. Since running migrations outside a transaction can be dangerous, consider performing very few operations in migrations that add concurrent indexes. We recommend to run migrations with concurrent indexes in isolation and disable those features only temporarily. Index types When creating an index, the index type can be specified with the :using option. The :using option can be an atom or a string, and its value is passed to the generated USING clause as-is. For example, PostgreSQL supports several index types like B-tree (the default), Hash, GIN, and GiST. More information on index types can be found in the PostgreSQL docs. Partial indexes Databases like PostgreSQL and MSSQL support partial indexes. A partial index is an index built over a subset of a table. The subset is defined by a conditional expression using the :where option. The :where option can be an atom or a string; its value is passed to the generated WHERE clause as-is. More information on partial indexes can be found in the PostgreSQL docs. Examples # With no name provided, the name of the below index defaults to # products_category_id_sku_index create index(&quot;products&quot;, [:category_id, :sku], unique: true) # The name can also be set explicitly drop index(&quot;products&quot;, [:category_id, :sku], name: :my_special_name) # Indexes can be added concurrently create index(&quot;products&quot;, [:category_id, :sku], concurrently: true) # The index type can be specified create index(&quot;products&quot;, [:name], using: :hash) # Partial indexes are created by specifying a :where option create index(&quot;products&quot;, [:user_id], where: &quot;price = 0&quot;, name: :free_products_index) Indexes also support custom expressions. Some databases may require the index expression to be written between parentheses: # Create an index on a custom expression create index(&quot;products&quot;, [&quot;(lower(name))&quot;], name: :products_lower_name_index) # Create a tsvector GIN index on PostgreSQL create index(&quot;products&quot;, [&quot;(to_tsvector(&#39;english&#39;, name))&quot;], name: :products_name_vector, using: &quot;GIN&quot;)"},{"ref":"Ecto.Migration.html#modify/3","title":"Ecto.Migration.modify/3","type":"function","doc":"Modifies the type of a column when altering a table. This command is not reversible unless the :from option is provided. If the :from value is a %Reference{}, the adapter will try to drop the corresponding foreign key constraints before modifying the type. See add/3 for more information on supported types. Examples alter table(&quot;posts&quot;) do modify :title, :text end Options :null - determines whether the column accepts null values. :default - changes the default value of the column. :from - specifies the current type of the column. :size - specifies the size of the type (for example, the number of characters). The default is no size. :precision - the precision for a numeric type. Required when :scale is specified. :scale - the scale of a numeric type. Defaults to 0."},{"ref":"Ecto.Migration.html#prefix/0","title":"Ecto.Migration.prefix/0","type":"function","doc":"Gets the migrator prefix."},{"ref":"Ecto.Migration.html#references/2","title":"Ecto.Migration.references/2","type":"function","doc":"Defines a foreign key. Examples create table(&quot;products&quot;) do add :group_id, references(&quot;groups&quot;) end Options :name - The name of the underlying reference, which defaults to &quot;\#{table}_\#{column}_fkey&quot;. :column - The foreign key column name, which defaults to :id. :prefix - The prefix for the reference. Defaults to the reference of the table if present, or nil. :type - The foreign key type, which defaults to :bigserial. :on_delete - What to do if the referenced entry is deleted. May be :nothing (default), :delete_all, :nilify_all, or :restrict. :on_update - What to do if the referenced entry is updated. May be :nothing (default), :update_all, :nilify_all, or :restrict."},{"ref":"Ecto.Migration.html#remove/1","title":"Ecto.Migration.remove/1","type":"function","doc":"Removes a column when altering a table. This command is not reversible as Ecto does not know what type it should add the column back as. See remove/3 as a reversible alternative. Examples alter table(&quot;posts&quot;) do remove :title end"},{"ref":"Ecto.Migration.html#remove/3","title":"Ecto.Migration.remove/3","type":"function","doc":"Removes a column in a reversible way when altering a table. type and opts are exactly the same as in add/3, and they are used when the command is reversed. If the type value is a %Reference{}, it is used to remove the constraint. Examples alter table(&quot;posts&quot;) do remove :title, :string, default: &quot;&quot; end"},{"ref":"Ecto.Migration.html#remove_if_exists/2","title":"Ecto.Migration.remove_if_exists/2","type":"function","doc":"Removes a column only if the column exists when altering the constraint if the reference type is passed once it only has the constraint name on reference structure. This command is not reversible as Ecto does not know about column existense before the removal attempt. Examples alter table(&quot;posts&quot;) do remove_if_exists :title, :string end"},{"ref":"Ecto.Migration.html#rename/2","title":"Ecto.Migration.rename/2","type":"function","doc":"Renames a table. Examples rename table(&quot;posts&quot;), to: table(&quot;new_posts&quot;)"},{"ref":"Ecto.Migration.html#rename/3","title":"Ecto.Migration.rename/3","type":"function","doc":"Renames a column outside of the alter statement. Examples rename table(&quot;posts&quot;), :title, to: :summary"},{"ref":"Ecto.Migration.html#repo/0","title":"Ecto.Migration.repo/0","type":"function","doc":"Gets the migrator repo."},{"ref":"Ecto.Migration.html#table/2","title":"Ecto.Migration.table/2","type":"function","doc":"Returns a table struct that can be given to create/2, alter/2, drop/1, etc. Examples create table(&quot;products&quot;) do add :name, :string add :price, :decimal end drop table(&quot;products&quot;) create table(&quot;products&quot;, primary_key: false) do add :name, :string add :price, :decimal end Options :primary_key - when false, a primary key field is not generated on table creation. :engine - customizes the table storage for supported databases. For MySQL, the default is InnoDB. :prefix - the prefix for the table. :options - provide custom options that will be appended after the generated statement. For example, &quot;WITH&quot;, &quot;INHERITS&quot;, or &quot;ON COMMIT&quot; clauses."},{"ref":"Ecto.Migration.html#timestamps/1","title":"Ecto.Migration.timestamps/1","type":"function","doc":"Adds :inserted_at and :updated_at timestamp columns. Those columns are of :naive_datetime type and by default cannot be null. A list of opts can be given to customize the generated fields. Options :inserted_at - the name of the column for storing insertion times. Setting it to false disables the column. :updated_at - the name of the column for storing last-updated-at times. Setting it to false disables the column. :type - the type of the :inserted_at and :updated_at columns. Defaults to :naive_datetime."},{"ref":"Ecto.Migration.html#unique_index/3","title":"Ecto.Migration.unique_index/3","type":"function","doc":"Shortcut for creating a unique index. See index/3 for more information."},{"ref":"Ecto.Migrator.html","title":"Ecto.Migrator","type":"module","doc":"This module provides the migration API. Example defmodule MyApp.MigrationExample do use Ecto.Migration def up do execute &quot;CREATE TABLE users(id serial PRIMARY_KEY, username text)&quot; end def down do execute &quot;DROP TABLE users&quot; end end Ecto.Migrator.up(Repo, 20080906120000, MyApp.MigrationExample)"},{"ref":"Ecto.Migrator.html#down/4","title":"Ecto.Migrator.down/4","type":"function","doc":"Runs a down migration on the given repository. Options :log - the level to use for logging. Defaults to :info. Can be any of Logger.level/0 values or a boolean. :log_sql - the level to use for logging of SQL instructions. Defaults to false. Can be any of Logger.level/0 values or a boolean. :prefix - the prefix to run the migrations on :dynamic_repo - the name of the Repo supervisor process. See Ecto.Repo.put_dynamic_repo/1."},{"ref":"Ecto.Migrator.html#migrated_versions/2","title":"Ecto.Migrator.migrated_versions/2","type":"function","doc":"Gets all migrated versions. This function ensures the migration table exists if no table has been defined yet. Options :prefix - the prefix to run the migrations on :dynamic_repo - the name of the Repo supervisor process. See Ecto.Repo.put_dynamic_repo/1."},{"ref":"Ecto.Migrator.html#migrations/1","title":"Ecto.Migrator.migrations/1","type":"function","doc":"Returns an array of tuples as the migration status of the given repo, without actually running any migrations. Equivalent to: Ecto.Migrator.migrations(repo, Ecto.Migrator.migrations_path(repo))"},{"ref":"Ecto.Migrator.html#migrations/2","title":"Ecto.Migrator.migrations/2","type":"function","doc":"Returns an array of tuples as the migration status of the given repo, without actually running any migrations."},{"ref":"Ecto.Migrator.html#migrations_path/1","title":"Ecto.Migrator.migrations_path/1","type":"function","doc":"Gets the migrations path from a repository."},{"ref":"Ecto.Migrator.html#run/3","title":"Ecto.Migrator.run/3","type":"function","doc":"Runs migrations for the given repository. Equivalent to: Ecto.Migrator.run(repo, Ecto.Migrator.migrations_path(repo), direction, opts) See run/4 for more information."},{"ref":"Ecto.Migrator.html#run/4","title":"Ecto.Migrator.run/4","type":"function","doc":"Apply migrations to a repository with a given strategy. The second argument identifies where the migrations are sourced from. A binary representing a directory may be passed, in which case we will load all files following the &quot;\#{VERSION}_\#{NAME}.exs&quot; schema. The migration_source may also be a list of a list of tuples that identify the version number and migration modules to be run, for example: Ecto.Migrator.run(Repo, [{0, MyApp.Migration1}, {1, MyApp.Migration2}, ...], :up, opts) A strategy (which is one of :all, :step or :to) must be given as an option. Execution model In order to run migrations, at least two database connections are necessary. One is used to lock the &quot;schema_migrations&quot; table and the other one to effectively run the migrations. This allows multiple nodes to run migrations at the same time, but guarantee that only one of them will effectively migrate the database. A downside of this approach is that migrations cannot run dynamically during test under the Ecto.Adapters.SQL.Sandbox, as the sandbox has to share a single connection across processes to guarantee the changes can be reverted. Options :all - runs all available if true :step - runs the specific number of migrations :to - runs all until the supplied version is reached :log - the level to use for logging. Defaults to :info. Can be any of Logger.level/0 values or a boolean. :prefix - the prefix to run the migrations on"},{"ref":"Ecto.Migrator.html#up/4","title":"Ecto.Migrator.up/4","type":"function","doc":"Runs an up migration on the given repository. Options :log - the level to use for logging of migration instructions. Defaults to :info. Can be any of Logger.level/0 values or a boolean. :log_sql - the level to use for logging of SQL instructions. Defaults to false. Can be any of Logger.level/0 values or a boolean. :prefix - the prefix to run the migrations on :dynamic_repo - the name of the Repo supervisor process. See Ecto.Repo.put_dynamic_repo/1. :strict_version_order - abort when applying a migration with old timestamp"},{"ref":"Ecto.Migrator.html#with_repo/3","title":"Ecto.Migrator.with_repo/3","type":"function","doc":"Ensures the repo is started to perform migration operations. All of the application required to run the repo will be started before hand with chosen mode. If the repo has not yet been started, it is manually started, with a :pool_size of 2, before the given function is executed, and the repo is then terminated. If the repo was already started, then the function is directly executed, without terminating the repo afterwards. Although this function was designed to start repositories for running migrations, it can be used by any code, Mix task, or release tooling that needs to briefly start a repository to perform a certain operation and then terminate. The repo may also configure a :start_apps_before_migration option which is a list of applications to be started before the migration runs. It returns {:ok, fun_return, apps}, with all apps that have been started, or {:error, term}. Options :pool_size - The pool size to start the repo for migrations. Defaults to 2. :mode - The mode to start all applications. Defaults to :permanent. Examples {:ok, _, _} = Ecto.Migrator.with_repo(repo, fn repo -&gt; Ecto.Migrator.run(repo, :up, all: true) end)"},{"ref":"Mix.Tasks.Ecto.Dump.html","title":"Mix.Tasks.Ecto.Dump","type":"task","doc":"Dumps the current environment&#39;s database structure for the given repository into a structure file. The repository must be set under :ecto_repos in the current app configuration or given via the -r option. This task needs some shell utility to be present on the machine running the task. DatabaseUtility needed PostgreSQLpg_dump MySQLmysqldump Example mix ecto.dump Command line options -r, --repo - the repo to load the structure info from -d, --dump-path - the path of the dump file to create -q, --quiet - run the command quietly --no-compile - does not compile applications before dumping --no-deps-check - does not check depedendencies before dumping"},{"ref":"Mix.Tasks.Ecto.Gen.Migration.html","title":"Mix.Tasks.Ecto.Gen.Migration","type":"task","doc":"Generates a migration. The repository must be set under :ecto_repos in the current app configuration or given via the -r option. Examples mix ecto.gen.migration add_posts_table mix ecto.gen.migration add_posts_table -r Custom.Repo The generated migration filename will be prefixed with the current timestamp in UTC which is used for versioning and ordering. By default, the migration will be generated to the &quot;priv/YOUR_REPO/migrations&quot; directory of the current application but it can be configured to be any subdirectory of priv by specifying the :priv key under the repository configuration. This generator will automatically open the generated file if you have ECTO_EDITOR set in your environment variable. Command line options -r, --repo - the repo to generate migration for --no-compile - does not compile applications before running --no-deps-check - does not check depedendencies before running Configuration If the current app configuration specifies a custom migration module the generated migration code will use that rather than the default Ecto.Migration: config :ecto_sql, migration_module: MyApplication.CustomMigrationModule"},{"ref":"Mix.Tasks.Ecto.Load.html","title":"Mix.Tasks.Ecto.Load","type":"task","doc":"Loads the current environment&#39;s database structure for the given repository from a previously dumped structure file. The repository must be set under :ecto_repos in the current app configuration or given via the -r option. This task needs some shell utility to be present on the machine running the task. DatabaseUtility needed PostgreSQLpsql MySQLmysql Example mix ecto.load Command line options -r, --repo - the repo to load the structure info into -d, --dump-path - the path of the dump file to load from -q, --quiet - run the command quietly -f, --force - do not ask for confirmation when loading data. Configuration is asked only when :start_permanent is set to true (typically in production) --no-compile - does not compile applications before loading --no-deps-check - does not check depedendencies before loading --skip-if-loaded - does not load the dump file if the repo has the migrations table up"},{"ref":"Mix.Tasks.Ecto.Migrate.html","title":"Mix.Tasks.Ecto.Migrate","type":"task","doc":"Runs the pending migrations for the given repository. Migrations are expected at &quot;priv/YOUR_REPO/migrations&quot; directory of the current application, where &quot;YOUR_REPO&quot; is the last segment in your repository name. For example, the repository MyApp.Repo will use &quot;priv/repo/migrations&quot;. The repository Whatever.MyRepo will use &quot;priv/my_repo/migrations&quot;. You can configure a repository to use another directory by specifying the :priv key under the repository configuration. The &quot;migrations&quot; part will be automatically appended to it. For instance, to use &quot;priv/custom_repo/migrations&quot;: config :my_app, MyApp.Repo, priv: &quot;priv/custom_repo&quot; This task runs all pending migrations by default. To migrate up to a specific version number, supply --to version_number. To migrate a specific number of times, use --step n. The repositories to migrate are the ones specified under the :ecto_repos option in the current app configuration. However, if the -r option is given, it replaces the :ecto_repos config. Since Ecto tasks can only be executed once, if you need to migrate multiple repositories, set :ecto_repos accordingly or pass the -r flag multiple times. If a repository has not yet been started, one will be started outside your application supervision tree and shutdown afterwards. Examples mix ecto.migrate mix ecto.migrate -r Custom.Repo mix ecto.migrate -n 3 mix ecto.migrate --step 3 mix ecto.migrate -v 20080906120000 mix ecto.migrate --to 20080906120000 Command line options -r, --repo - the repo to migrate --all - run all pending migrations --step / -n - run n number of pending migrations --to - run all migrations up to and including version --quiet - do not log migration commands --prefix - the prefix to run migrations on --pool-size - the pool size if the repository is started only for the task (defaults to 2) --log-sql - log the raw sql migrations are running --strict-version-order - abort when applying a migration with old timestamp --no-compile - does not compile applications before migrating --no-deps-check - does not check depedendencies before migrating --migrations-path - the path to run the migrations from"},{"ref":"Mix.Tasks.Ecto.Migrations.html","title":"Mix.Tasks.Ecto.Migrations","type":"task","doc":"Displays the up / down migration status for the given repository. The repository must be set under :ecto_repos in the current app configuration or given via the -r option. By default, migrations are expected at &quot;priv/YOUR_REPO/migrations&quot; directory of the current application but it can be configured by specifying the :priv key under the repository configuration. If the repository has not been started yet, one will be started outside our application supervision tree and shutdown afterwards. Examples mix ecto.migrations mix ecto.migrations -r Custom.Repo Command line options -r, --repo - the repo to obtain the status for --no-compile - does not compile applications before running --no-deps-check - does not check depedendencies before running --migrations-path - the path to run the migrations from"},{"ref":"Mix.Tasks.Ecto.Rollback.html","title":"Mix.Tasks.Ecto.Rollback","type":"task","doc":"Reverts applied migrations in the given repository. Migrations are expected at &quot;priv/YOUR_REPO/migrations&quot; directory of the current application, where &quot;YOUR_REPO&quot; is the last segment in your repository name. For example, the repository MyApp.Repo will use &quot;priv/repo/migrations&quot;. The repository Whatever.MyRepo will use &quot;priv/my_repo/migrations&quot;. You can configure a repository to use another directory by specifying the :priv key under the repository configuration. The &quot;migrations&quot; part will be automatically appended to it. For instance, to use &quot;priv/custom_repo/migrations&quot;: config :my_app, MyApp.Repo, priv: &quot;priv/custom_repo&quot; This task runs all pending migrations by default. Runs the last applied migration by default. To roll back to a version number, supply --to version_number. To roll back a specific number of times, use --step n. To undo all applied migrations, provide --all. The repositories to rollback are the ones specified under the :ecto_repos option in the current app configuration. However, if the -r option is given, it replaces the :ecto_repos config. If a repository has not yet been started, one will be started outside your application supervision tree and shutdown afterwards. Examples mix ecto.rollback mix ecto.rollback -r Custom.Repo mix ecto.rollback -n 3 mix ecto.rollback --step 3 mix ecto.rollback --to 20080906120000 Command line options -r, --repo - the repo to rollback --all - revert all applied migrations --step / -n - revert n number of applied migrations --to - revert all migrations down to and including version --quiet - do not log migration commands --prefix - the prefix to run migrations on --pool-size - the pool size if the repository is started only for the task (defaults to 2) --log-sql - log the raw sql migrations are running --no-compile - does not compile applications before rolling back --no-deps-check - does not check depedendencies before rolling back --migrations-path - the path to run the migrations from"},{"ref":"Ecto.Adapters.MySQL.html","title":"Ecto.Adapters.MySQL","type":"module","doc":"Adapter module for MySQL. It uses Mariaex for communicating to the database. Currently it supports old MySQL versions but upcoming Ecto releases will require 5.7+. Options MySQL options split in different categories described below. All options can be given via the repository configuration: Connection options :hostname - Server hostname :socket_dir - Connect to MySQL via UNIX sockets in the given directory The socket name is derived based on the port. This is the preferred method for configuring sockets and it takes precedence over the hostname. If you are connecting to a socket outside of the Postgres convention, use :socket instead; :socket - Connect to MySQL via UNIX sockets in the given path. This option takes precedence over the :hostname and :socket_dir :port - Server port (default: 3306) :username - Username :password - User password :database - the database to connect to :pool - The connection pool module, defaults to DBConnection.ConnectionPool :ssl - Set to true if ssl should be used (default: false) :ssl_opts - A list of ssl options, see Erlang&#39;s ssl docs :parameters - Keyword list of connection parameters :connect_timeout - The timeout for establishing new connections (default: 5000) :socket_options - Specifies socket configuration :cli_protocol - The protocol used for the mysql client connection (default: tcp). This option is only used for mix ecto.load and mix ecto.dump, via the mysql command. For more information, please check MySQL docs The :socket_options are particularly useful when configuring the size of both send and receive buffers. For example, when Ecto starts with a pool of 20 connections, the memory usage may quickly grow from 20MB to 50MB based on the operating system default values for TCP buffers. It is advised to stick with the operating system defaults but they can be tweaked if desired: socket_options: [recbuf: 8192, sndbuf: 8192] We also recommend developers to consult the Mariaex.start_link/1 documentation for a complete listing of all supported options. Storage options :charset - the database encoding (default: &quot;utf8&quot;) :collation - the collation order :dump_path - where to place dumped structures After connect callback If you want to execute a callback as soon as connection is established to the database, you can use the :after_connect configuration. For example, in your repository configuration you can add: after_connect: {Mariaex, :query!, [&quot;SET variable = value&quot;, []]} You can also specify your own module that will receive the Mariaex connection as argument. Limitations There are some limitations when using Ecto with MySQL that one needs to be aware of. Engine Since Ecto uses transactions, MySQL users running old versions (5.1 and before) must ensure their tables use the InnoDB engine as the default (MyISAM) does not support transactions. Tables created by Ecto are guaranteed to use InnoDB, regardless of the MySQL version. UUIDs MySQL does not support UUID types. Ecto emulates them by using binary(16). Read after writes Because MySQL does not support RETURNING clauses in INSERT and UPDATE, it does not support the :read_after_writes option of Ecto.Schema.field/3. DDL Transaction MySQL does not support migrations inside transactions as it automatically commits after some commands like CREATE TABLE. Therefore MySQL migrations does not run inside transactions. usec in datetime Old MySQL versions did not support usec in datetime while more recent versions would round or truncate the usec value. Therefore, in case the user decides to use microseconds in datetimes and timestamps with MySQL, be aware of such differences and consult the documentation for your MySQL version. If your version of MySQL supports microsecond precision, you will be able to utilize Ecto&#39;s usec types."},{"ref":"Ecto.Adapters.MyXQL.html","title":"Ecto.Adapters.MyXQL","type":"module","doc":"Adapter module for MySQL. It uses MyXQL for communicating to the database. Options MySQL options split in different categories described below. All options can be given via the repository configuration: Connection options :protocol - Set to :socket for using UNIX domain socket, or :tcp for TCP (default: :socket) :socket - Connect to MySQL via UNIX sockets in the given path. :hostname - Server hostname :port - Server port (default: 3306) :username - Username :password - User password :database - the database to connect to :pool - The connection pool module, defaults to DBConnection.ConnectionPool :ssl - Set to true if ssl should be used (default: false) :ssl_opts - A list of ssl options, see Erlang&#39;s ssl docs :connect_timeout - The timeout for establishing new connections (default: 5000) :cli_protocol - The protocol used for the mysql client connection (default: &quot;tcp&quot;). This option is only used for mix ecto.load and mix ecto.dump, via the mysql command. For more information, please check MySQL docs :socket_options - Specifies socket configuration The :socket_options are particularly useful when configuring the size of both send and receive buffers. For example, when Ecto starts with a pool of 20 connections, the memory usage may quickly grow from 20MB to 50MB based on the operating system default values for TCP buffers. It is advised to stick with the operating system defaults but they can be tweaked if desired: socket_options: [recbuf: 8192, sndbuf: 8192] We also recommend developers to consult the MyXQL.start_link/1 documentation for a complete listing of all supported options. Storage options :charset - the database encoding (default: &quot;utf8&quot;) :collation - the collation order :dump_path - where to place dumped structures After connect callback If you want to execute a callback as soon as connection is established to the database, you can use the :after_connect configuration. For example, in your repository configuration you can add: after_connect: {MyXQL, :query!, [&quot;SET variable = value&quot;, []]} You can also specify your own module that will receive the MyXQL connection as argument. Limitations There are some limitations when using Ecto with MySQL that one needs to be aware of. Engine Tables created by Ecto are guaranteed to use InnoDB, regardless of the MySQL version. UUIDs MySQL does not support UUID types. Ecto emulates them by using binary(16). Read after writes Because MySQL does not support RETURNING clauses in INSERT and UPDATE, it does not support the :read_after_writes option of Ecto.Schema.field/3. DDL Transaction MySQL does not support migrations inside transactions as it automatically commits after some commands like CREATE TABLE. Therefore MySQL migrations does not run inside transactions. usec in datetime Old MySQL versions did not support usec in datetime while more recent versions would round or truncate the usec value. Therefore, in case the user decides to use microseconds in datetimes and timestamps with MySQL, be aware of such differences and consult the documentation for your MySQL version. If your version of MySQL supports microsecond precision, you will be able to utilize Ecto&#39;s usec types."},{"ref":"Ecto.Adapters.Postgres.html","title":"Ecto.Adapters.Postgres","type":"module","doc":"Adapter module for PostgreSQL. It uses Postgrex for communicating to the database. Features Full query support (including joins, preloads and associations) Support for transactions Support for data migrations Support for ecto.create and ecto.drop operations Support for transactional tests via Ecto.Adapters.SQL Options Postgres options split in different categories described below. All options can be given via the repository configuration: config :your_app, YourApp.Repo, ... Connection options :hostname - Server hostname :socket_dir - Connect to Postgres via UNIX sockets in the given directory The socket name is derived based on the port. This is the preferred method for configuring sockets and it takes precedence over the hostname. If you are connecting to a socket outside of the Postgres convention, use :socket instead; :socket - Connect to Postgres via UNIX sockets in the given path. This option takes precedence over the :hostname and :socket_dir :username - Username :password - User password :port - Server port (default: 5432) :database - the database to connect to :maintenance_database - Specifies the name of the database to connect to when creating or dropping the database. Defaults to &quot;postgres&quot; :pool - The connection pool module, defaults to DBConnection.ConnectionPool :ssl - Set to true if ssl should be used (default: false) :ssl_opts - A list of ssl options, see Erlang&#39;s ssl docs :parameters - Keyword list of connection parameters :connect_timeout - The timeout for establishing new connections (default: 5000) :prepare - How to prepare queries, either :named to use named queries or :unnamed to force unnamed queries (default: :named) :socket_options - Specifies socket configuration The :socket_options are particularly useful when configuring the size of both send and receive buffers. For example, when Ecto starts with a pool of 20 connections, the memory usage may quickly grow from 20MB to 50MB based on the operating system default values for TCP buffers. It is advised to stick with the operating system defaults but they can be tweaked if desired: socket_options: [recbuf: 8192, sndbuf: 8192] We also recommend developers to consult the Postgrex.start_link/1 documentation for a complete listing of all supported options. Storage options :encoding - the database encoding (default: &quot;UTF8&quot;) :template - the template to create the database from :lc_collate - the collation order :lc_ctype - the character classification :dump_path - where to place dumped structures After connect callback If you want to execute a callback as soon as connection is established to the database, you can use the :after_connect configuration. For example, in your repository configuration you can add: after_connect: {Postgrex, :query!, [&quot;SET search_path TO global_prefix&quot;, []]} You can also specify your own module that will receive the Postgrex connection as argument. Extensions Both PostgreSQL and its adapter for Elixir, Postgrex, support an extension system. If you want to use custom extensions for Postgrex alongside Ecto, you must define a type module with your extensions. Create a new file anywhere in your application with the following: Postgrex.Types.define(MyApp.PostgresTypes, [MyExtension.Foo, MyExtensionBar] ++ Ecto.Adapters.Postgres.extensions()) Once your type module is defined, you can configure the repository to use it: config :my_app, MyApp.Repo, types: MyApp.PostgresTypes"},{"ref":"Ecto.Adapters.Postgres.html#extensions/0","title":"Ecto.Adapters.Postgres.extensions/0","type":"function","doc":"All Ecto extensions for Postgrex."},{"ref":"Ecto.Adapter.Migration.html","title":"Ecto.Adapter.Migration","type":"behaviour","doc":"Specifies the adapter migrations API."},{"ref":"Ecto.Adapter.Migration.html#c:execute_ddl/3","title":"Ecto.Adapter.Migration.execute_ddl/3","type":"callback","doc":"Executes migration commands."},{"ref":"Ecto.Adapter.Migration.html#c:lock_for_migrations/4","title":"Ecto.Adapter.Migration.lock_for_migrations/4","type":"callback","doc":"Locks the migrations table and emits the locked versions for callback execution. It returns the result of calling the given function with a list of versions."},{"ref":"Ecto.Adapter.Migration.html#c:supports_ddl_transaction?/0","title":"Ecto.Adapter.Migration.supports_ddl_transaction?/0","type":"callback","doc":"Checks if the adapter supports ddl transaction."},{"ref":"Ecto.Adapter.Migration.html#t:adapter_meta/0","title":"Ecto.Adapter.Migration.adapter_meta/0","type":"type","doc":""},{"ref":"Ecto.Adapter.Migration.html#t:command/0","title":"Ecto.Adapter.Migration.command/0","type":"type","doc":"All migration commands"},{"ref":"Ecto.Adapter.Migration.html#t:ddl_object/0","title":"Ecto.Adapter.Migration.ddl_object/0","type":"type","doc":"A struct that represents a table or index in a database schema. These database objects can be modified through the use of a Data Definition Language, hence the name DDL object."},{"ref":"Ecto.Adapter.Migration.html#t:table_subcommand/0","title":"Ecto.Adapter.Migration.table_subcommand/0","type":"type","doc":"All commands allowed within the block passed to table/2"},{"ref":"Ecto.Adapter.Structure.html","title":"Ecto.Adapter.Structure","type":"behaviour","doc":"Specifies the adapter structure (dump/load) API."},{"ref":"Ecto.Adapter.Structure.html#c:structure_dump/2","title":"Ecto.Adapter.Structure.structure_dump/2","type":"callback","doc":"Dumps the given structure. The path will be looked in the config under :dump_path or default to the structure path inside default. Returns :ok if it was dumped successfully, an error tuple otherwise. Examples structure_dump(&quot;priv/repo&quot;, username: &quot;postgres&quot;, database: &quot;ecto_test&quot;, hostname: &quot;localhost&quot;)"},{"ref":"Ecto.Adapter.Structure.html#c:structure_load/2","title":"Ecto.Adapter.Structure.structure_load/2","type":"callback","doc":"Loads the given structure. The path will be looked in the config under :dump_path or default to the structure path inside default. Returns :ok if it was loaded successfully, an error tuple otherwise. Examples structure_load(&quot;priv/repo&quot;, username: &quot;postgres&quot;, database: &quot;ecto_test&quot;, hostname: &quot;localhost&quot;)"},{"ref":"Ecto.Adapters.SQL.Connection.html","title":"Ecto.Adapters.SQL.Connection","type":"behaviour","doc":"Specifies the behaviour to be implemented by all SQL connections."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:all/1","title":"Ecto.Adapters.SQL.Connection.all/1","type":"callback","doc":"Receives a query and must return a SELECT query."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:child_spec/1","title":"Ecto.Adapters.SQL.Connection.child_spec/1","type":"callback","doc":"Receives options and returns DBConnection supervisor child specification."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:ddl_logs/1","title":"Ecto.Adapters.SQL.Connection.ddl_logs/1","type":"callback","doc":"Receives a query result and returns a list of logs."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:delete/4","title":"Ecto.Adapters.SQL.Connection.delete/4","type":"callback","doc":"Returns a DELETE for the filters returning the given returning."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:delete_all/1","title":"Ecto.Adapters.SQL.Connection.delete_all/1","type":"callback","doc":"Receives a query and must return a DELETE query."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:execute/4","title":"Ecto.Adapters.SQL.Connection.execute/4","type":"callback","doc":"Executes a cached query."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:execute_ddl/1","title":"Ecto.Adapters.SQL.Connection.execute_ddl/1","type":"callback","doc":"Receives a DDL command and returns a query that executes it."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:insert/6","title":"Ecto.Adapters.SQL.Connection.insert/6","type":"callback","doc":"Returns an INSERT for the given rows in table returning the given returning."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:prepare_execute/5","title":"Ecto.Adapters.SQL.Connection.prepare_execute/5","type":"callback","doc":"Prepares and executes the given query with DBConnection."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:query/4","title":"Ecto.Adapters.SQL.Connection.query/4","type":"callback","doc":"Runs the given statement as query."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:stream/4","title":"Ecto.Adapters.SQL.Connection.stream/4","type":"callback","doc":"Returns a stream that prepares and executes the given query with DBConnection."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:table_exists_query/1","title":"Ecto.Adapters.SQL.Connection.table_exists_query/1","type":"callback","doc":"Returns a queryable to check if the given table exists."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:to_constraints/1","title":"Ecto.Adapters.SQL.Connection.to_constraints/1","type":"callback","doc":"Receives the exception returned by query/4. The constraints are in the keyword list and must return the constraint type, like :unique, and the constraint name as a string, for example: [unique: &quot;posts_title_index&quot;] Must return an empty list if the error does not come from any constraint."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:update/5","title":"Ecto.Adapters.SQL.Connection.update/5","type":"callback","doc":"Returns an UPDATE for the given fields in table filtered by filters returning the given returning."},{"ref":"Ecto.Adapters.SQL.Connection.html#c:update_all/1","title":"Ecto.Adapters.SQL.Connection.update_all/1","type":"callback","doc":"Receives a query and values to update and must return an UPDATE query."},{"ref":"Ecto.Adapters.SQL.Connection.html#t:cached/0","title":"Ecto.Adapters.SQL.Connection.cached/0","type":"type","doc":"The cached query which is a DBConnection Query"},{"ref":"Ecto.Adapters.SQL.Connection.html#t:connection/0","title":"Ecto.Adapters.SQL.Connection.connection/0","type":"type","doc":""},{"ref":"Ecto.Adapters.SQL.Connection.html#t:name/0","title":"Ecto.Adapters.SQL.Connection.name/0","type":"type","doc":"The query name"},{"ref":"Ecto.Adapters.SQL.Connection.html#t:params/0","title":"Ecto.Adapters.SQL.Connection.params/0","type":"type","doc":""},{"ref":"Ecto.Adapters.SQL.Connection.html#t:statement/0","title":"Ecto.Adapters.SQL.Connection.statement/0","type":"type","doc":"The SQL statement"},{"ref":"Ecto.Migration.Command.html","title":"Ecto.Migration.Command","type":"module","doc":"Used internally by adapters. This represents the up and down legs of a reversible raw command that is usually defined with Ecto.Migration.execute/1. To define a reversible command in a migration, see Ecto.Migration.execute/2."},{"ref":"Ecto.Migration.Command.html#t:t/0","title":"Ecto.Migration.Command.t/0","type":"type","doc":""},{"ref":"Ecto.Migration.Constraint.html","title":"Ecto.Migration.Constraint","type":"module","doc":"Used internally by adapters. To define a constraint in a migration, see Ecto.Migration.constraint/3."},{"ref":"Ecto.Migration.Constraint.html#t:t/0","title":"Ecto.Migration.Constraint.t/0","type":"type","doc":""},{"ref":"Ecto.Migration.Index.html","title":"Ecto.Migration.Index","type":"module","doc":"Used internally by adapters. To define an index in a migration, see Ecto.Migration.index/3."},{"ref":"Ecto.Migration.Index.html#t:t/0","title":"Ecto.Migration.Index.t/0","type":"type","doc":""},{"ref":"Ecto.Migration.Reference.html","title":"Ecto.Migration.Reference","type":"module","doc":"Used internally by adapters. To define a reference in a migration, see Ecto.Migration.references/2."},{"ref":"Ecto.Migration.Reference.html#t:t/0","title":"Ecto.Migration.Reference.t/0","type":"type","doc":""},{"ref":"Ecto.Migration.Table.html","title":"Ecto.Migration.Table","type":"module","doc":"Used internally by adapters. To define a table in a migration, see Ecto.Migration.table/2."},{"ref":"Ecto.Migration.Table.html#t:t/0","title":"Ecto.Migration.Table.t/0","type":"type","doc":""}]